@@ -232,6 +232,7 @@ void WayfireBatteryInfo::init(Gtk::HBox *container)
     }
 
     button_box.add(icon);
+    button.get_style_context()->add_class("battery");
     button.get_style_context()->add_class("flat");
 
     status_opt.set_callback([=] () { update_details(); });
@@ -245,7 +246,6 @@ void WayfireBatteryInfo::init(Gtk::HBox *container)
 
     container->pack_start(button, Gtk::PACK_SHRINK);
     button_box.add(label);
-    button_box.set_spacing(5);
 
     button.add(button_box);
     button.property_scale_factor().signal_changed()
