@@ -10,29 +10,67 @@
 #include <gtkmm/flowbox.h>
 #include <gtkmm/scrolledwindow.h>
 #include <gtkmm/separator.h>
+#include <gtkmm/eventbox.h>
 #include <set>
 
 class WayfireMenu;
 using AppInfo = Glib::RefPtr<Gio::AppInfo>;
 
-class WfMenuMenuItem : public Gtk::HBox
+class WfMenuCategory
 {
   public:
-    WfMenuMenuItem(WayfireMenu *menu, AppInfo app);
+    WfMenuCategory(std::string name, std::string icon_name);
+    std::string get_name();
+    std::string get_icon_name();
+    std::vector<Glib::RefPtr<Gio::DesktopAppInfo>> items;
 
-    bool matches(Glib::ustring text);
-    bool fuzzy_match(Glib::ustring text);
+  private:
+    std::string name;
+    std::string icon_name;
+};
+
+class WfMenuCategoryButton : public Gtk::Button
+{
+  public:
+    WfMenuCategoryButton(WayfireMenu *menu, std::string category, std::string label, std::string icon_name);
+
+  private:
+    WayfireMenu *menu;
+    Gtk::HBox m_box;
+    Gtk::Label m_label;
+    Gtk::Image m_image;
+
+    std::string category;
+    std::string label;
+    std::string icon_name;
+    void on_click();
+};
+
+class WfMenuMenuItem : public Gtk::FlowBoxChild
+{
+  public:
+    WfMenuMenuItem(WayfireMenu *menu, Glib::RefPtr<Gio::DesktopAppInfo> app);
+
+    uint32_t matches(Glib::ustring text);
+    uint32_t fuzzy_match(Glib::ustring text);
     bool operator <(const WfMenuMenuItem& other);
+    void set_search_value(uint32_t value);
+    uint32_t get_search_value();
 
   private:
     WayfireMenu *menu;
     Gtk::Box m_left_pad, m_right_pad;
-    Gtk::Button m_button;
+    Gtk::HBox m_padding_box;
     Gtk::VBox m_button_box;
+    Gtk::HBox m_list_box;
     Gtk::Image m_image;
     Gtk::Label m_label;
+    Gtk::Menu m_action_menu;
 
-    AppInfo m_app_info;
+    bool m_has_actions = false;
+    uint32_t m_search_value = 0;
+
+    Glib::RefPtr<Gio::DesktopAppInfo> m_app_info;
     void on_click();
 };
 
@@ -77,20 +115,27 @@ class WayfireLogoutUI
     void on_cancel_click();
 };
 
+class WayfireMenuInjectionEntry : public Gtk::Entry
+{
+  public:
+    bool inject(GdkEventKey *ev);
+};
+
 class WayfireMenu : public WayfireWidget
 {
     WayfireOutput *output;
 
     Gtk::Box flowbox_container;
-    Gtk::HBox hbox, hbox_bottom;
+    Gtk::HBox hbox, hbox_bottom, scroll_pair;
     Gtk::VBox bottom_pad;
     Gtk::VBox popover_layout_box;
+    Gtk::VBox category_box;
     Gtk::Separator separator;
     Gtk::Image main_image;
-    Gtk::Entry search_box;
+    WayfireMenuInjectionEntry search_box;
     Gtk::FlowBox flowbox;
     Gtk::Button logout_button;
-    Gtk::ScrolledWindow scrolled_window;
+    Gtk::ScrolledWindow app_scrolled_window, category_scrolled_window;
     std::unique_ptr<WayfireMenuButton> button;
     std::unique_ptr<WayfireLogoutUI> logout_ui;
 
@@ -101,8 +146,11 @@ class WayfireMenu : public WayfireWidget
     void load_menu_items_from_dir(std::string directory);
     void load_menu_items_all();
 
+    void add_category_app(std::string category, Glib::RefPtr<Gio::DesktopAppInfo>);
+
     bool update_icon();
 
+    bool m_sort_names     = true;
     bool fuzzy_filter     = false;
     int32_t count_matches = 0;
 
@@ -111,27 +159,45 @@ class WayfireMenu : public WayfireWidget
     void on_search_changed();
     void on_popover_shown();
 
-    std::vector<std::unique_ptr<WfMenuMenuItem>> items;
     /* loaded_apps is a list of the already-opened applications + their execs,
      * so that we don't show duplicate entries */
     std::set<std::pair<std::string, std::string>> loaded_apps;
+    std::unordered_map<std::string, std::unique_ptr<WfMenuCategory>> category_list;
+    std::string category = "All";
+    std::vector<std::string> category_order = {
+        "All", "Network", "Education", "Office", "Development", "Graphics", "AudioVideo", "Game", "Science",
+        "Settings", "System", "Utility", "Hidden"
+    };
 
     WfOption<std::string> menu_logout_command{"panel/menu_logout_command"};
     WfOption<bool> fuzzy_search_enabled{"panel/menu_fuzzy_search"};
     WfOption<std::string> panel_position{"panel/position"};
     WfOption<std::string> menu_icon{"panel/menu_icon"};
     WfOption<int> menu_size{"panel/launchers_size"};
-    WfOption<int> menu_min_content_width{"panel/menu_min_content_width"};
+    WfOption<int> menu_min_category_width{"panel/menu_min_category_width"};
     WfOption<int> menu_min_content_height{"panel/menu_min_content_height"};
+    WfOption<bool> menu_show_categories{"panel/menu_show_categories"};
     void update_popover_layout();
+    void update_category_width();
+    void update_content_height();
+    void update_content_width();
     void create_logout_ui();
     void on_logout_click();
+    void key_press_search();
+    void select_first_flowbox_item();
+    void set_default_to_selection();
 
   public:
     void init(Gtk::HBox *container) override;
+    void populate_menu_items(std::string category);
+    void populate_menu_categories();
     void toggle_menu();
     void hide_menu();
     void refresh();
+    void set_category(std::string category);
+    WfOption<bool> menu_list{"panel/menu_list"};
+    WfOption<int> menu_min_content_width{"panel/menu_min_content_width"};
+
     WayfireMenu(WayfireOutput *output)
     {
         this->output = output;
