@@ -97,9 +97,21 @@ WfSingleNotification::WfSingleNotification(const Notification & notification)
 
     child.add(top_bar);
 
+    Gtk::IconSize body_image_size = Gtk::ICON_SIZE_DIALOG;
     if (notification.hints.image_data)
     {
-        image.set(notification.hints.image_data);
+        int width;
+        int height;
+        Gtk::IconSize::lookup(body_image_size, width, height);
+
+        auto image_pixbuf = notification.hints.image_data;
+        if (image_pixbuf->get_width() > width)
+        {
+            image_pixbuf = image_pixbuf->scale_simple(width,
+                width * image_pixbuf->get_height() / image_pixbuf->get_width(), Gdk::INTERP_BILINEAR);
+        }
+
+        image.set(image_pixbuf);
     } else if (!notification.hints.image_path.empty())
     {
         if (is_file_uri(notification.hints.image_path))
@@ -107,7 +119,7 @@ WfSingleNotification::WfSingleNotification(const Notification & notification)
             image.set(path_from_uri(notification.hints.image_path));
         } else
         {
-            image.set_from_icon_name(notification.hints.image_path, Gtk::ICON_SIZE_DIALOG);
+            image.set_from_icon_name(notification.hints.image_path, body_image_size);
         }
     }
 
