@@ -2,7 +2,7 @@
 
 void WayfireStatusNotifier::init(Gtk::HBox *container)
 {
-    icons_hbox.set_spacing(5);
+    icons_hbox.get_style_context()->add_class("tray");
     container->add(icons_hbox);
 }
 
