@@ -11,6 +11,7 @@
 #include <gdk/gdkwayland.h>
 #include <cmath>
 
+#include <glibmm.h>
 #include "toplevel.hpp"
 #include "gtk-utils.hpp"
 #include "panel.hpp"
@@ -40,6 +41,7 @@ class WayfireToplevel::impl
     Gtk::Menu menu;
     Gtk::MenuItem minimize, maximize, close;
     Glib::RefPtr<Gtk::GestureDrag> drag_gesture;
+    sigc::connection m_drag_timeout;
 
     Glib::ustring app_id, title;
 
@@ -55,10 +57,10 @@ class WayfireToplevel::impl
         zwlr_foreign_toplevel_handle_v1_add_listener(handle,
             &toplevel_handle_v1_impl, this);
 
+        button.get_style_context()->add_class("window-button");
         button_contents.add(image);
         button_contents.add(label);
         button_contents.set_halign(Gtk::ALIGN_START);
-        button_contents.set_spacing(5);
         button.add(button_contents);
         button.set_tooltip_text("none");
 
@@ -86,6 +88,29 @@ class WayfireToplevel::impl
         menu.attach_to_widget(button);
         menu.show_all();
 
+        button.drag_dest_set(Gtk::DEST_DEFAULT_MOTION & Gtk::DEST_DEFAULT_HIGHLIGHT, Gdk::DragAction(0));
+
+        button.signal_drag_motion().connect(
+            [this] (const Glib::RefPtr<Gdk::DragContext>, gint x, gint y, guint time) -> bool
+        {
+            if (!m_drag_timeout)
+            {
+                m_drag_timeout = Glib::signal_timeout().connect(sigc::mem_fun(this,
+                    &WayfireToplevel::impl::drag_paused), 700);
+            }
+
+            return true;
+        });
+
+        button.signal_drag_leave().connect(
+            [this] (const Glib::RefPtr<Gdk::DragContext>, guint time)
+        {
+            if (m_drag_timeout)
+            {
+                m_drag_timeout.disconnect();
+            }
+        });
+
         drag_gesture = Gtk::GestureDrag::create(button);
         drag_gesture->signal_drag_begin().connect_notify(
             sigc::mem_fun(this, &WayfireToplevel::impl::on_drag_begin));
@@ -102,6 +127,14 @@ class WayfireToplevel::impl
     double grab_abs_start_x;
     bool drag_exceeds_threshold;
 
+    bool drag_paused()
+    {
+        auto gseat = Gdk::Display::get_default()->get_default_seat();
+        auto seat  = gdk_wayland_seat_get_wl_seat(gseat->gobj());
+        zwlr_foreign_toplevel_handle_v1_activate(handle, seat);
+        return false;
+    }
+
     void on_drag_begin(double _x, double _y)
     {
         auto& container = window_list->box;
@@ -109,7 +142,7 @@ class WayfireToplevel::impl
         grab_start_x = _x;
         grab_start_y = _y;
 
-        set_flat_class(false);
+        set_classes(state);
         window_list->box.set_top_widget(&button);
 
         /* Find the distance between pointer X and button origin */
@@ -161,7 +194,7 @@ class WayfireToplevel::impl
         int height = button.get_allocated_height();
 
         window_list->box.set_top_widget(nullptr);
-        set_flat_class(!(state & WF_TOPLEVEL_STATE_ACTIVATED));
+        set_classes(state);
 
         /* When a button is dropped after dnd, we ignore the unclick
          * event so action doesn't happen in addition to dropping.
@@ -187,7 +220,7 @@ class WayfireToplevel::impl
         {
             if (event->button == 3)
             {
-                menu.popup(event->button, event->time);
+                menu.popup_at_widget(&button, Gdk::GRAVITY_NORTH, Gdk::GRAVITY_SOUTH, NULL);
                 return true; // It has been handled.
             } else if ((event->button == 2) && middle_click_close)
             {
@@ -427,26 +460,49 @@ class WayfireToplevel::impl
         }
     }
 
-    void set_flat_class(bool on)
+    void set_classes(uint32_t state)
     {
-        if (on)
+        if (state & WF_TOPLEVEL_STATE_ACTIVATED)
         {
-            button.get_style_context()->add_class("flat");
+            button.get_style_context()->add_class("activated");
+            button.get_style_context()->remove_class("flat");
         } else
         {
-            button.get_style_context()->remove_class("flat");
+            button.get_style_context()->add_class("flat");
+            button.get_style_context()->remove_class("activated");
+        }
+
+        if (state & WF_TOPLEVEL_STATE_MINIMIZED)
+        {
+            button.get_style_context()->add_class("minimized");
+        } else
+        {
+            button.get_style_context()->remove_class("minimized");
+        }
+
+        if (state & WF_TOPLEVEL_STATE_MAXIMIZED)
+        {
+            button.get_style_context()->add_class("maximized");
+        } else
+        {
+            button.get_style_context()->remove_class("maximized");
         }
     }
 
     void set_state(uint32_t state)
     {
         this->state = state;
-        set_flat_class(!(state & WF_TOPLEVEL_STATE_ACTIVATED));
+        set_classes(state);
         update_menu_item_text();
     }
 
     ~impl()
     {
+        if (m_drag_timeout)
+        {
+            m_drag_timeout.disconnect();
+        }
+
         zwlr_foreign_toplevel_handle_v1_destroy(handle);
     }
 
@@ -665,11 +721,7 @@ Icon get_from_desktop_app_info(std::string app_id)
 
     std::vector<std::string> prefixes = {
         "",
-        "/usr/share/applications/",
-        "/usr/share/applications/kde/",
-        "/usr/share/applications/org.kde.",
-        "/usr/local/share/applications/",
-        "/usr/local/share/applications/org.kde.",
+        "org.kde.",
     };
 
     std::vector<std::string> app_id_variations = {
@@ -690,8 +742,7 @@ Icon get_from_desktop_app_info(std::string app_id)
             {
                 if (!app_info)
                 {
-                    app_info = Gio::DesktopAppInfo
-                        ::create_from_filename(prefix + id + suffix);
+                    app_info = Gio::DesktopAppInfo::create(prefix + id + suffix);
                 }
             }
         }
