@@ -2,6 +2,7 @@
 #define WF_GTK_UTILS
 
 #include <gtkmm/image.h>
+#include <gtkmm/icontheme.h>
 #include <gtkmm/cssprovider.h>
 #include <string>
 
@@ -14,16 +15,18 @@ Glib::RefPtr<Gtk::CssProvider> load_css_from_path(std::string path);
 struct WfIconLoadOptions
 {
     int user_scale = -1;
-    bool invert = false;
+    bool invert    = false;
 };
 
 /* Sets the content of the image to the pixbuf, applying device scale factor "scale" */
-void set_image_pixbuf(Gtk::Image &image, Glib::RefPtr<Gdk::Pixbuf> pixbuf, int scale);
+void set_image_pixbuf(Gtk::Image & image, Glib::RefPtr<Gdk::Pixbuf> pixbuf, int scale);
 
 /* Sets the content of the image to the corresponding icon from the default theme,
  * using the given options */
 void set_image_icon(Gtk::Image& image, std::string icon_name, int size,
-                    const WfIconLoadOptions& options);
+    const WfIconLoadOptions& options = {},
+    const Glib::RefPtr<Gtk::IconTheme>& icon_theme
+    = Gtk::IconTheme::get_default());
 
 void invert_pixbuf(Glib::RefPtr<Gdk::Pixbuf>& pbuff);
 
